@@ -23,7 +23,7 @@
 #include <fplll/enum/enumerate_base.h>
 #include <fplll/enum/enumerate_ext.h>
 #include <fplll/enum/evaluator.h>
-#include <fplll/gso.h>
+#include <fplll/gso_interface.h>
 #include <memory>
 
 FPLLL_BEGIN_NAMESPACE
@@ -31,11 +31,12 @@ FPLLL_BEGIN_NAMESPACE
 template <typename ZT, typename FT> class EnumerationDyn : public EnumerationBase
 {
 public:
-  EnumerationDyn(MatGSO<ZT, FT> &gso, Evaluator<FT> &evaluator,
+  EnumerationDyn(MatGSOInterface<ZT, FT> &gso, Evaluator<FT> &evaluator,
                  const vector<int> &max_indices = vector<int>())
       : _gso(gso), _evaluator(evaluator)
   {
     _max_indices = max_indices;
+    std::fill(nodes.begin(), nodes.end(), 0);
   }
 
   void enumerate(int first, int last, FT &fmaxdist, long fmaxdistexpo,
@@ -44,10 +45,19 @@ public:
                  const vector<enumf> &pruning = vector<enumf>(), bool dual = false,
                  bool subtree_reset = false);
 
-  inline uint64_t get_nodes() const { return nodes; }
+  inline uint64_t get_nodes(const int level = -1) const
+  {
+    if (level == -1)
+    {
+      return std::accumulate(nodes.cbegin(), nodes.cend(), 0);
+    }
+    return nodes[level];
+  }
+
+  inline array<uint64_t, FPLLL_EXTENUM_MAX_EXTENUM_DIM> get_nodes_array() { return nodes; }
 
 private:
-  MatGSO<ZT, FT> &_gso;
+  MatGSOInterface<ZT, FT> &_gso;
   Evaluator<FT> &_evaluator;
   vector<FT> target;
 
@@ -68,9 +78,9 @@ private:
 template <typename ZT, typename FT> class Enumeration
 {
 public:
-  Enumeration(MatGSO<ZT, FT> &gso, Evaluator<FT> &evaluator,
+  Enumeration(MatGSOInterface<ZT, FT> &gso, Evaluator<FT> &evaluator,
               const vector<int> &max_indices = vector<int>())
-      : _gso(gso), _evaluator(evaluator), _max_indices(max_indices), enumdyn(nullptr), _nodes(0)
+      : _gso(gso), _evaluator(evaluator), _max_indices(max_indices), enumdyn(nullptr), _nodes{}
   {
   }
 
@@ -87,7 +97,7 @@ public:
         enumext.reset(new ExternalEnumeration<ZT, FT>(_gso, _evaluator));
       if (enumext->enumerate(first, last, fmaxdist, fmaxdistexpo, pruning, dual))
       {
-        _nodes = enumext->get_nodes();
+        _nodes = enumext->get_nodes_array();
         return;
       }
     }
@@ -97,18 +107,25 @@ public:
       enumdyn.reset(new EnumerationDyn<ZT, FT>(_gso, _evaluator, _max_indices));
     enumdyn->enumerate(first, last, fmaxdist, fmaxdistexpo, target_coord, subtree, pruning, dual,
                        subtree_reset);
-    _nodes = enumdyn->get_nodes();
+    _nodes = enumdyn->get_nodes_array();
   }
 
-  inline uint64_t get_nodes() const { return _nodes; }
+  inline uint64_t get_nodes(const int level = -1) const
+  {
+    if (level == -1)
+      return std::accumulate(_nodes.begin(), _nodes.end(), 0);
+    return _nodes[level];
+  }
+
+  inline array<uint64_t, FPLLL_EXTENUM_MAX_EXTENUM_DIM> get_nodes_array() const { return _nodes; }
 
 private:
-  MatGSO<ZT, FT> &_gso;
+  MatGSOInterface<ZT, FT> &_gso;
   Evaluator<FT> &_evaluator;
   vector<int> _max_indices;
   std::unique_ptr<EnumerationDyn<ZT, FT>> enumdyn;
   std::unique_ptr<ExternalEnumeration<ZT, FT>> enumext;
-  uint64_t _nodes;
+  array<uint64_t, FPLLL_EXTENUM_MAX_EXTENUM_DIM> _nodes;
 };
 
 FPLLL_END_NAMESPACE
