@@ -39,7 +39,9 @@ enum Action
   ACTION_HKZ,
   ACTION_BKZ,
   ACTION_SVP,
-  ACTION_CVP
+  ACTION_CVP,
+  ACTION_HLLL,
+  ACTION_PRU
 };
 
 struct Options
@@ -48,7 +50,7 @@ struct Options
       : action(ACTION_LLL), method(LM_WRAPPER), int_type(ZT_MPZ), float_type(FT_DEFAULT),
         delta(LLL_DEF_DELTA), eta(LLL_DEF_ETA), precision(0), early_red(false), siegel(false),
         no_lll(false), block_size(0), bkz_gh_factor(1.1), verbose(false), input_file(NULL),
-        output_format(NULL)
+        output_format(NULL), theta(HLLL_DEF_THETA), c(HLLL_DEF_C)
   {
     bkz_flags     = 0;
     bkz_max_loops = 0;
@@ -68,6 +70,10 @@ struct Options
   int block_size;
   int bkz_flags;
   int bkz_max_loops;
+  int prune_start;
+  int prune_end;
+  double prune_pre_nodes;
+  double prune_min_prob;
   double bkz_max_time;
   string bkz_dump_gso_filename;
   double bkz_gh_factor;
@@ -76,6 +82,9 @@ struct Options
   bool verbose;
   const char *input_file;
   const char *output_format;
+
+  double theta;
+  double c;
 };
 
 #endif
