@@ -17,6 +17,7 @@
 #include <gso.h>
 #include <gso_gram.h>
 #include <gso_interface.h>
+#include <householder.h>
 #include <nr/matrix.h>
 //#include <random>
 #include <test_utils.h>
@@ -69,6 +70,87 @@ template <class ZT, class FT> bool rs_are_equal(MatGSO<ZT, FT> M1, MatGSOGram<ZT
   return true;
 }
 
+template <class FT> void assert_diag_R(FT rhd, int i, int &status)
+{
+  if (rhd.cmp(0.0) <= 0)
+  {
+    cerr << "R(" << i << ", " << i << ") must be positive." << endl;
+    status = 1;
+  }
+}
+
+template <class FT> void assert_mu_r_householder(FT rh, FT rhd, FT mu, FT r, int &status)
+{
+  if (abs(mu - rh / rhd) > 0.001)
+  {
+    cerr << "Error (mu): " << mu << " != " << rh / rhd << endl;
+    status = 1;
+  }
+  if (abs(r - rh * rhd) > 0.001)
+  {
+    cerr << "Error (r): " << r << " != " << rh * rhd << endl;
+    status = 1;
+  }
+}
+
+/**
+ * Test if:
+ *   mu(i, j) = R(i, j) / R(j, j)
+ *   r(i, j) = R(i, j) * R(j, j)
+ */
+template <class ZT, class FT> int test_householder(ZZ_mat<ZT> &A)
+{
+  int status = 0;
+  ZZ_mat<ZT> U;
+  ZZ_mat<ZT> UT;
+  MatGSO<Z_NR<ZT>, FP_NR<FT>> M(A, U, UT, GSO_INT_GRAM);
+  // Since HOUSEHOLDER_DEFAULT, the exponent returned by get_R(_naively) must be equal to zero.
+  MatHouseholder<Z_NR<ZT>, FP_NR<FT>> Mhouseholder(A, U, UT, HOUSEHOLDER_DEFAULT);
+  M.update_gso();
+  // Here, we just need to refresh R. However, refresh_R() does not modify n_known_rows, but
+  // refresh_R_bf() does.
+  Mhouseholder.refresh_R_bf();
+  Mhouseholder.update_R();
+  Mhouseholder.update_R_naively();
+
+  FP_NR<FT> r;
+  FP_NR<FT> rh;
+  FP_NR<FT> rhd;
+  FP_NR<FT> mu;
+  long expo;
+
+  for (int i = 0; i < A.get_rows(); i++)
+  {
+    Mhouseholder.get_R(rhd, i, i, expo);
+    FPLLL_CHECK(expo == 0, "expo must be equal to 0");
+    assert_diag_R(rhd, i, status);
+    Mhouseholder.get_R_naively(rhd, i, i, expo);
+    FPLLL_CHECK(expo == 0, "expo must be equal to 0");
+    assert_diag_R(rhd, i, status);
+  }
+
+  for (int i = 0; i < A.get_rows(); i++)
+  {
+    for (int j = 0; j < i; j++)
+    {
+      M.get_r(r, i, j);
+      M.get_mu(mu, i, j);
+      Mhouseholder.get_R(rh, i, j, expo);
+      FPLLL_CHECK(expo == 0, "expo must be equal to 0");
+      Mhouseholder.get_R(rhd, j, j, expo);
+      FPLLL_CHECK(expo == 0, "expo must be equal to 0");
+      assert_mu_r_householder(rh, rhd, mu, r, status);
+      Mhouseholder.get_R_naively(rh, i, j, expo);
+      FPLLL_CHECK(expo == 0, "expo must be equal to 0");
+      Mhouseholder.get_R_naively(rhd, j, j, expo);
+      FPLLL_CHECK(expo == 0, "expo must be equal to 0");
+      assert_mu_r_householder(rh, rhd, mu, r, status);
+    }
+  }
+
+  return status;
+}
+
 template <class ZT, class FT> int test_ggso(ZZ_mat<ZT> &A)
 {
   // TEST A
@@ -151,8 +233,8 @@ template <class ZT, class FT> int test_ggso(ZZ_mat<ZT> &A)
 template <class ZT, class FT> int test_filename(const char *input_filename)
 {
   ZZ_mat<ZT> A;
-  int status = read_matrix(A, input_filename);
-  // if status == 1, read_matrix fails.
+  int status = read_file(A, input_filename);
+  // if status == 1, read_file fails.
   if (status == 1)
   {
     return 1;
@@ -166,14 +248,17 @@ template <class ZT, class FT> int test_filename(const char *input_filename)
   }
   if (retvalue & 2)
   {
-    cerr << input_filename << " shows different GSO-outputs for grammatrix representation and "
-                              "basis representation after moving rows.\n";
+    cerr << input_filename
+         << " shows different GSO-outputs for grammatrix representation and "
+            "basis representation after moving rows.\n";
   }
   if (retvalue & 4)
   {
-    cerr << input_filename << " shows different GSO-outputs for grammatrix representation and "
-                              "basis representation after adding rows.\n";
+    cerr << input_filename
+         << " shows different GSO-outputs for grammatrix representation and "
+            "basis representation after adding rows.\n";
   }
+  retvalue |= test_householder<ZT, FT>(A);
   if (retvalue > 0)
   {
     return 1;
@@ -210,6 +295,10 @@ template <class ZT, class FT> int test_int_rel(int d, int b)
         << " shows different GSO-outputs for grammatrix representation and basis representation.\n";
     return 1;
   }
+  retvalue |= test_householder<ZT, FT>(A);
+  if (retvalue > 0)
+    return 1;
+
   return 0;
 }
 
@@ -219,6 +308,7 @@ int main(int /*argc*/, char ** /*argv*/)
   int status = 0;
 
   status |= test_filename<mpz_t, double>(TESTDATADIR "/tests/lattices/example2_in");
+  status |= test_filename<mpz_t, double>(TESTDATADIR "/tests/lattices/example3_in");
   status |= test_filename<mpz_t, double>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice");
   status |= test_filename<mpz_t, double>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice2");
   status |= test_filename<mpz_t, double>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice3");
@@ -228,6 +318,7 @@ int main(int /*argc*/, char ** /*argv*/)
   status |= test_int_rel<mpz_t, double>(40, 10);
 
   status |= test_filename<mpz_t, mpfr_t>(TESTDATADIR "/tests/lattices/example2_in");
+  status |= test_filename<mpz_t, mpfr_t>(TESTDATADIR "/tests/lattices/example3_in");
   status |= test_filename<mpz_t, mpfr_t>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice");
   status |= test_filename<mpz_t, mpfr_t>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice2");
   status |= test_filename<mpz_t, mpfr_t>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice3");
@@ -238,6 +329,7 @@ int main(int /*argc*/, char ** /*argv*/)
 
 #ifdef FPLLL_WITH_LONG_DOUBLE
   status |= test_filename<mpz_t, long double>(TESTDATADIR "/tests/lattices/example2_in");
+  status |= test_filename<mpz_t, long double>(TESTDATADIR "/tests/lattices/example3_in");
   status |= test_filename<mpz_t, long double>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice");
   status |=
       test_filename<mpz_t, long double>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice2");
@@ -252,6 +344,7 @@ int main(int /*argc*/, char ** /*argv*/)
 #endif
 #ifdef FPLLL_WITH_QD
   status |= test_filename<mpz_t, dd_real>(TESTDATADIR "/tests/lattices/example2_in");
+  status |= test_filename<mpz_t, dd_real>(TESTDATADIR "/tests/lattices/example3_in");
   status |= test_filename<mpz_t, dd_real>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice");
   status |= test_filename<mpz_t, dd_real>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice2");
   status |= test_filename<mpz_t, dd_real>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice3");
@@ -261,6 +354,7 @@ int main(int /*argc*/, char ** /*argv*/)
   status |= test_int_rel<mpz_t, dd_real>(40, 10);
 
   status |= test_filename<mpz_t, qd_real>(TESTDATADIR "/tests/lattices/example2_in");
+  status |= test_filename<mpz_t, qd_real>(TESTDATADIR "/tests/lattices/example3_in");
   status |= test_filename<mpz_t, qd_real>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice");
   status |= test_filename<mpz_t, qd_real>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice2");
   status |= test_filename<mpz_t, qd_real>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice3");
@@ -271,6 +365,7 @@ int main(int /*argc*/, char ** /*argv*/)
 #endif
 #ifdef FPLLL_WITH_DPE
   status |= test_filename<mpz_t, dpe_t>(TESTDATADIR "/tests/lattices/example2_in");
+  status |= test_filename<mpz_t, dpe_t>(TESTDATADIR "/tests/lattices/example3_in");
   status |= test_filename<mpz_t, dpe_t>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice");
   status |= test_filename<mpz_t, dpe_t>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice2");
   status |= test_filename<mpz_t, dpe_t>(TESTDATADIR "/tests/lattices/example_cvp_in_lattice3");
